import 'package:flutter/material.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_router.dart';
import 'widget/widget_avator.dart';
import 'widget/widget_divider.dart';

///
///联系人搜索页面
///
class ContactFinder extends StatefulWidget {
  const ContactFinder({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _ContactFinderState();
}

class _ContactFinderState extends State<ContactFinder> {
  final _valueController = TextEditingController();
  final _findResList = <Object>[];

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['ContactFinder.initState()']);
  }

  @override
  void dispose() {
    AppConfig.logDebug(['ContactFinder.dispose()']);
    _valueController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Scaffold(
      appBar: _buildAppBar(themeData),
      body: _buildBody(themeData),
    );
  }

  AppBar _buildAppBar(ThemeData themeData) {
    return AppBar(
      centerTitle: false,
      titleSpacing: 0,
      title: TextFormField(
        autofocus: true,
        controller: _valueController,
        keyboardType: TextInputType.emailAddress,
        textInputAction: TextInputAction.search,
        maxLength: Validator.shortTextLenMax,
        decoration: InputDecoration(
          contentPadding: EdgeInsets.zero,
          border: InputBorder.none,
          hintText: AppLocalizations.contactFinderInputHint,
          counterText: '',
        ),
        onEditingComplete: _onFindSureBtnPressed,
      ),
      actions: [
        TextButton(
          onPressed: _onFindSureBtnPressed,
          child: Text(AppLocalizations.contactFinderActionBtnSure),
        ),
      ],
    );
  }

  Widget _buildBody(ThemeData themeData) {
    return Column(
      children: [
        AppConfig.cardGlobalSpaceMedium,
        Expanded(
          child: ListView.builder(
            physics: AppConfig.globalScrollPhysics,
            itemCount: _findResList.length,
            itemBuilder: (context, index) {
              final item = _findResList[index];
              if (item is User) {
                return Column(
                  children: [
                    index == 0 ? WidgetDivider.forFull : const SizedBox(),
                    ListTile(
                      tileColor: themeData.cardColor,
                      leading: WidgetAvator(type: WidgetAvatorType.small, url: item.icon, urlList: item.head, onFormatUrl: AppConfig.formatAttachUrl),
                      title: Text(ComTools.formatUserNick(item), overflow: TextOverflow.ellipsis),
                      trailing: AppConfig.infoListTileTailArrowIC,
                      onTap: () => AppRouter.pushContactUserInfoPage(context, uid: item.id, fid: AppConfig.client.user.id, from: Constant.shipFromSearch),
                    ),
                    WidgetDivider.forFull,
                  ],
                );
              } else if (item is String) {
                return ListTile(
                  dense: true,
                  title: Text(item, style: themeData.textTheme.caption, textAlign: TextAlign.center),
                );
              } else {
                return const SizedBox();
              }
            },
          ),
        ),
        AppConfig.cardGlobalSpaceMedium,
      ],
    );
  }

  void _onFindSureBtnPressed() async {
    SmartDialog.showLoading(msg: AppLocalizations.contactFinderSureLoading);
    final result = await AppConfig.client.userSearch(keywords: _valueController.text.trim());
    SmartDialog.dismiss();
    if (result.ok) {
      if (mounted) {
        setState(() {
          _findResList.clear();
          _findResList.addAll(result.extra!);
          if (_findResList.isEmpty) {
            _findResList.add(AppLocalizations.contactFinderItemResEmptyText);
          }
        });
      }
    } else {
      SmartDialog.showToast(result.desc);
    }
  }
}
