import 'package:flutter/material.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_router.dart';
import 'widget/widget_badge.dart';

///
/// 联系人页面
///
class ContactHome extends StatefulWidget {
  const ContactHome({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _ContactHomeState();
}

class _ContactHomeState extends State<ContactHome> {
  int _currentIndex = 0;

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['ContactHome.initState()']);
    AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: false);
  }

  @override
  void dispose() {
    AppConfig.logDebug(['ContactHome.dispose()']);
    AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: true);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    final Widget iconButton;
    final Widget bodyWidget;
    switch (_currentIndex) {
      case 0:
        iconButton = IconButton(
          icon: const Icon(Icons.search),
          tooltip: AppLocalizations.contactHomeActionBtnUserSearchTip,
          onPressed: () => AppRouter.pushContactFinderPage(context),
        );
        bodyWidget = AppRouter.createContactHomeUserShipsWidget();
        break;
      case 1:
        iconButton = IconButton(
          icon: const Icon(Icons.add),
          tooltip: AppLocalizations.contactHomeActionBtnTeamCreateTip,
          onPressed: _onTeamCreateBtnPressed,
        );
        bodyWidget = AppRouter.createContactHomeTeamShipsWidget();
        break;
      case 2:
        iconButton = IconButton(
          icon: const Icon(Icons.search),
          tooltip: AppLocalizations.contactHomeActionBtnUserSearchTip,
          onPressed: () => AppRouter.pushContactFinderPage(context),
        );
        bodyWidget = AppRouter.createContactHomeWaitShipsWidget();
        break;
      default:
        iconButton = const SizedBox();
        bodyWidget = Center(child: Text('$_currentIndex'));
        break;
    }
    return Scaffold(
      resizeToAvoidBottomInset: false,
      appBar: AppBar(
        backgroundColor: _currentIndex == 2 ? null : themeData.cardColor,
        title: Text([
          AppLocalizations.contactHomeTabItemUserships,
          AppLocalizations.contactHomeTabItemTeamships,
          AppLocalizations.contactHomeTabItemWaitships,
        ][_currentIndex]),
        actions: [iconButton],
      ),
      body: bodyWidget,
      bottomNavigationBar: BottomNavigationBar(
        items: [
          BottomNavigationBarItem(
            icon: const Icon(Icons.people_alt_outlined),
            activeIcon: const Icon(Icons.people_alt),
            label: AppLocalizations.contactHomeTabItemUserships,
          ),
          BottomNavigationBarItem(
            icon: const Icon(Icons.groups_outlined),
            activeIcon: const Icon(Icons.groups),
            label: AppLocalizations.contactHomeTabItemTeamships,
          ),
          BottomNavigationBarItem(
            icon: WidgetBadge(icon: const Icon(Icons.person_add_outlined), count: AppConfig.client.waitshipState.unread),
            activeIcon: WidgetBadge(icon: const Icon(Icons.person_add), count: AppConfig.client.waitshipState.unread),
            label: AppLocalizations.contactHomeTabItemWaitships,
          ),
        ],
        currentIndex: _currentIndex,
        onTap: (index) {
          if (mounted) {
            setState(() {
              _currentIndex = index;
            });
          }
        },
      ),
    );
  }

  void _onTeamCreateBtnPressed() {
    AppRouter.pushContactUserShipsPickerPage(context, title: AppLocalizations.contactHomeActionBtnTeamCreateTip).then((value) async {
      if (value == null) return;
      SmartDialog.showLoading(msg: AppLocalizations.contactHomeTeamCreateLoading);
      final result = await AppConfig.client.teamCreate(uids: value.map((e) => e.rid).toList());
      SmartDialog.dismiss();
      SmartDialog.showToast(result.desc);
      if (result.ok) {
        if (mounted) {
          setState(() {
            _currentIndex = 1;
          });
        }
      }
    });
  }

  void _clientCommonListener(EasyPacket packet) {
    if (mounted) {
      setState(() {});
    }
  }
}
