import 'package:flutter/material.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_router.dart';
import 'userme_editor.dart';
import 'widget/widget_avator.dart';
import 'widget/widget_badge.dart';
import 'widget/widget_divider.dart';
import 'widget/widget_picture.dart';
import 'widget/widget_sheet.dart';

///
///我的信息详情页
///
class UsermeDetail extends StatefulWidget {
  const UsermeDetail({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _UsermeDetailState();
}

class _UsermeDetailState extends State<UsermeDetail> {
  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['UsermeDetail.initState()']);
    AppConfig.client.setOnUserUpdateListener(_onUserUpdateListener, remove: false);
  }

  @override
  void dispose() {
    AppConfig.logDebug(['UsermeDetail.dispose()']);
    AppConfig.client.setOnUserUpdateListener(_onUserUpdateListener, remove: true);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Scaffold(
      appBar: _buildAppBar(themeData),
      body: _buildBody(themeData),
    );
  }

  AppBar _buildAppBar(ThemeData themeData) {
    return AppBar(
      title: Text(AppLocalizations.usermeDetailTitle),
    );
  }

  Widget _buildBody(ThemeData themeData) {
    return ListView(
      physics: AppConfig.globalScrollPhysics,
      children: [
        Ink(
          color: themeData.cardColor,
          child: Column(
            children: [
              WidgetDivider.forFull,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemNoTitle, style: themeData.textTheme.subtitle1),
                title: Align(
                  alignment: Alignment.centerRight,
                  child: WidgetAvator(type: WidgetAvatorType.small, url: AppConfig.client.user.icon, urlList: AppConfig.client.user.head, onFormatUrl: AppConfig.formatAttachUrl),
                ),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: _onListHeadItemTap,
              ),
              WidgetDivider.forListTileLeadingNone,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemNoTitle, style: themeData.textTheme.subtitle1),
                title: Text(AppConfig.client.user.no, textAlign: TextAlign.right, overflow: TextOverflow.ellipsis),
                trailing: WidgetBadge(count: AppConfig.client.user.no.isEmpty ? 1 : 0, dotOnly: true, align: WidgetBadgeAlign.centerLeft, margin: WidgetBadge.hugeMargin, icon: AppConfig.infoListTileTailArrowIC),
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemNoTitle, editorType: UsermeEditorType.no),
              ),
              WidgetDivider.forListTileLeadingNone,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemPwdTitle, style: themeData.textTheme.subtitle1),
                trailing: WidgetBadge(count: AppConfig.client.user.pwd.isEmpty ? 1 : 0, dotOnly: true, align: WidgetBadgeAlign.centerLeft, margin: WidgetBadge.hugeMargin, icon: AppConfig.infoListTileTailArrowIC),
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemPwdTitle, editorType: UsermeEditorType.pwd),
              ),
              WidgetDivider.forListTileLeadingNone,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemNickTitle, style: themeData.textTheme.subtitle1),
                title: Text(AppConfig.client.user.nick, textAlign: TextAlign.right, overflow: TextOverflow.ellipsis),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemNickTitle, editorType: UsermeEditorType.nick),
              ),
              WidgetDivider.forListTileLeadingNone,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemDescTitle, style: themeData.textTheme.subtitle1),
                title: Text(AppConfig.client.user.desc, textAlign: TextAlign.right, overflow: TextOverflow.ellipsis),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemDescTitle, editorType: UsermeEditorType.desc),
              ),
              WidgetDivider.forFull,
            ],
          ),
        ),
        AppConfig.cardGlobalSpaceMedium,
        Ink(
          color: themeData.cardColor,
          child: Column(
            children: [
              WidgetDivider.forFull,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemPhoneTitle, style: themeData.textTheme.subtitle1),
                title: Text(AppConfig.client.user.phone, textAlign: TextAlign.right, overflow: TextOverflow.ellipsis),
                trailing: WidgetBadge(count: AppConfig.client.user.phone.isEmpty ? 1 : 0, dotOnly: true, align: WidgetBadgeAlign.centerLeft, margin: WidgetBadge.hugeMargin, icon: AppConfig.infoListTileTailArrowIC),
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemPhoneTitle, editorType: UsermeEditorType.phone),
              ),
              WidgetDivider.forListTileLeadingNone,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemNameTitle, style: themeData.textTheme.subtitle1),
                title: Text(AppConfig.client.user.name, textAlign: TextAlign.right, overflow: TextOverflow.ellipsis),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemNameTitle, editorType: UsermeEditorType.name),
              ),
              WidgetDivider.forListTileLeadingNone,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemCardTitle, style: themeData.textTheme.subtitle1),
                title: Text(AppConfig.client.user.card, textAlign: TextAlign.right, overflow: TextOverflow.ellipsis),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemCardTitle, editorType: UsermeEditorType.card),
              ),
              WidgetDivider.forListTileLeadingNone,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemSexTitle, style: themeData.textTheme.subtitle1),
                title: Text(AppLocalizations.modelConstMapText(AppConfig.client.user.sex), textAlign: TextAlign.right, overflow: TextOverflow.ellipsis),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemSexTitle, editorType: UsermeEditorType.sex),
              ),
              WidgetDivider.forListTileLeadingNone,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemBirthTitle, style: themeData.textTheme.subtitle1),
                title: Text(AppConfig.client.user.birth, textAlign: TextAlign.right, overflow: TextOverflow.ellipsis),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemBirthTitle, editorType: UsermeEditorType.birth),
              ),
              WidgetDivider.forListTileLeadingNone,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                leading: Text(AppLocalizations.usermeDetailListItemAddrTitle, style: themeData.textTheme.subtitle1),
                title: Text(ComTools.formatUserShowArea(AppConfig.client.user), textAlign: TextAlign.right, overflow: TextOverflow.ellipsis),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeEditorPage(context, fieldTitle: AppLocalizations.usermeDetailListItemAddrTitle, editorType: UsermeEditorType.addr),
              ),
              WidgetDivider.forFull,
            ],
          ),
        ),
        AppConfig.cardGlobalSpaceMedium,
      ],
    );
  }

  void _onListHeadItemTap() {
    final itemDatas = <WidgetSheetItem>[];
    itemDatas.add(WidgetSheetItem(0, AppLocalizations.usermeDetailMenuItemAlbum));
    itemDatas.add(WidgetSheetItem(1, AppLocalizations.usermeDetailMenuItemCamera));
    WidgetSheet.showBottomTextSheet(context, itemDatas, cancelText: AppLocalizations.usermeDetailMenuItemCancel).then((value) {
      switch (value) {
        case 0:
          WidgetPicture.pickFromGallery(context).then((value) async {
            if (value == null) return;
            final bytes = await AppRouter.pushImageEditorPage(context, type: WidgetPictureType.editAsJpg, editTitle: AppLocalizations.usermeDetailHeadEditTitle, editBytes: value, cropOnly: true, outputResize: true);
            if (bytes == null) return;
            SmartDialog.showLoading(msg: AppLocalizations.usermeDetailSaveLoading);
            final result = await AppConfig.client.attachUpload(type: Constant.metaTypeForever, fileBytes: [bytes], mediaType: MediaType.parse(WidgetPicture.mediaTypeJpg));
            SmartDialog.dismiss();
            if (result.ok) {
              _onHeadImageUploaded(result.extra!.first.path);
            } else {
              SmartDialog.showToast(result.desc);
            }
          });
          break;
        case 1:
          WidgetPicture.pickFromCamera(context).then((value) async {
            if (value == null) return;
            final bytes = await AppRouter.pushImageEditorPage(context, type: WidgetPictureType.editAsJpg, editTitle: AppLocalizations.usermeDetailHeadEditTitle, editBytes: value, cropOnly: true, outputResize: true);
            if (bytes == null) return;
            SmartDialog.showLoading(msg: AppLocalizations.usermeDetailSaveLoading);
            final result = await AppConfig.client.attachUpload(type: Constant.metaTypeForever, fileBytes: [bytes], mediaType: MediaType.parse(WidgetPicture.mediaTypeJpg));
            SmartDialog.dismiss();
            if (result.ok) {
              _onHeadImageUploaded(result.extra!.first.path);
            } else {
              SmartDialog.showToast(result.desc);
            }
          });
          break;
      }
    });
  }

  void _onHeadImageUploaded(String path) async {
    final user = UserDirty();
    user.icon = path;
    SmartDialog.showLoading(msg: AppLocalizations.usermeDetailSaveLoading);
    final result = await AppConfig.client.userUpdate(uid: AppConfig.client.user.id, fields: user.data);
    SmartDialog.dismiss();
    SmartDialog.showToast(result.desc);
  }

  void _onUserUpdateListener(EasyPacket packet) {
    if (mounted) {
      setState(() {});
    }
  }
}
