import 'package:flutter/material.dart';
import 'package:flutter_font_icons/flutter_font_icons.dart';

///
///自定义表情符号输入键盘
///
class WidgetEmojis extends StatefulWidget {
  final Color? pageColor;
  final int gridColumn;
  final double gridPaddingV;
  final TextStyle gridItemStyle;
  final double typeBarHeight;
  final Color? typeBarColor;
  final double typeItemFontSize;
  final Color typeNormalColor;
  final Color typeActiveColor;
  final Color? dividerColor;
  final ScrollPhysics? pagePhysics;
  final ScrollPhysics? gridPhysics;
  final ScrollPhysics? typePhysics;
  final void Function(String emoji)? onEmojiSelected;
  final void Function()? onBackspaceTap;

  const WidgetEmojis({
    Key? key,
    this.pageColor,
    this.gridColumn = 7,
    this.gridPaddingV = 8,
    this.gridItemStyle = const TextStyle(fontSize: 26),
    this.typeBarHeight = 58,
    this.typeBarColor,
    this.typeItemFontSize = 26,
    required this.typeNormalColor,
    required this.typeActiveColor,
    this.dividerColor,
    this.pagePhysics,
    this.gridPhysics,
    this.typePhysics,
    this.onEmojiSelected,
    this.onBackspaceTap,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _WidgetEmojisState();
}

class _WidgetEmojisState extends State<WidgetEmojis> with SingleTickerProviderStateMixin {
  final _pageController = PageController(initialPage: 0);
  int _currentPage = 0;

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    _pageController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: <Widget>[
        Expanded(
          child: Container(
            color: widget.pageColor,
            child: PageView.builder(
              controller: _pageController,
              scrollDirection: Axis.horizontal,
              physics: widget.pagePhysics,
              itemCount: _widgetEmojisTypesData.length,
              itemBuilder: (context, pageIndex) {
                return GridView.builder(
                  scrollDirection: Axis.vertical,
                  padding: EdgeInsets.symmetric(vertical: widget.gridPaddingV),
                  physics: widget.gridPhysics,
                  gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: widget.gridColumn),
                  itemCount: _widgetEmojisTypesData[pageIndex].length,
                  itemBuilder: (context, gridIndex) {
                    String emoji = _widgetEmojisTypesData[pageIndex][gridIndex];
                    return IconButton(
                      icon: Text(emoji, style: widget.gridItemStyle),
                      onPressed: () => _onGridItemPressed(emoji),
                    );
                  },
                );
              },
              onPageChanged: _onPageViewChange,
            ),
          ),
        ),
        Divider(color: widget.dividerColor),
        Container(
          height: widget.typeBarHeight,
          color: widget.typeBarColor,
          child: Row(
            children: <Widget>[
              Expanded(
                child: ListView.builder(
                  scrollDirection: Axis.horizontal,
                  physics: widget.typePhysics,
                  itemCount: _widgetEmojisTypesIcon.length,
                  itemExtent: widget.typeBarHeight,
                  itemBuilder: (context, index) {
                    return MaterialButton(
                      padding: EdgeInsets.zero,
                      child: Icon(_widgetEmojisTypesIcon[index], size: widget.typeItemFontSize, color: index == _currentPage ? widget.typeActiveColor : widget.typeNormalColor),
                      onPressed: () => _onTypeItemPressed(index),
                    );
                  },
                ),
              ),
              SizedBox(
                width: widget.typeBarHeight,
                height: widget.typeBarHeight,
                child: MaterialButton(
                  padding: EdgeInsets.zero,
                  onPressed: _onBackspacePressed,
                  child: Icon(MaterialCommunityIcons.backspace, size: widget.typeItemFontSize, color: widget.typeNormalColor),
                ),
              ),
            ],
          ),
        ),
      ],
    );
  }

  void _onPageViewChange(int index) {
    if (mounted) {
      setState(() {
        _currentPage = index;
      });
    }
  }

  void _onTypeItemPressed(int index) {
    if (_pageController.hasClients) {
      _pageController.jumpToPage(index); //会触发_onPageViewChange刷新，无需再次手动刷新页面
    }
  }

  void _onGridItemPressed(String emoji) {
    if (widget.onEmojiSelected != null) {
      widget.onEmojiSelected!(emoji);
    }
  }

  void _onBackspacePressed() {
    if (widget.onBackspaceTap != null) {
      widget.onBackspaceTap!();
    }
  }
}

const List<String> _widgetEmojisSmileys = [
  '😀',
  '😬',
  '😁',
  '😂',
  '😃',
  '😄',
  '🤣',
  '😅',
  '😆',
  '😇',
  '😉',
  '😊',
  '😋',
  '😌',
  '😍',
  '😘',
  '🤪',
  '😜',
  '😝',
  '🤑',
  '😎',
  '🤓',
  '🧐',
  '🤠',
  '🤗',
  '🤡',
  '😏',
  '😒',
  '🙄',
  '🤨',
  '🤔',
  '🤫',
  '🤭',
  '🤥',
  '😳',
  '😞',
  '😟',
  '😠',
  '😡',
  '🤬',
  '😔',
  '🙁',
  '😣',
  '😖',
  '😫',
  '😩',
  '😤',
  '😮',
  '😱',
  '😯',
  '😥',
  '😪',
  '🤤',
  '😓',
  '😭',
  '🤩',
  '😵',
  '😲',
  '🤯',
  '🤐',
  '😷',
  '🤕',
  '🤒',
  '🤮',
  '🤢',
  '🤧',
  '😴',
  '💤',
  '😈',
  '👿',
  '👹',
  '👺',
  '💩',
  '👻',
  '💀',
  '👽',
  '🤖',
  '🎃'
];

const List<String> _widgetEmojisHands = [
  '👐',
  '🤲',
  '🙌',
  '👏',
  '🙏',
  '🤝',
  '👍',
  '👎',
  '👊',
  '✊',
  '🤛',
  '🤜',
  '🤞',
  '✌',
  '🤘',
  '🤟',
  '👌',
  '👈',
  '👉',
  '👆',
  '👇',
  '☝',
  '✋',
  '🤚',
  '🖐',
  '🖖',
  '👋',
  '🤙',
  '💪',
  '🖕',
  '✍',
  '🤳',
  '💅',
  '👄',
  '👅',
  '👂',
  '👃',
  '👁',
  '👀',
  '🧠',
  '👤',
  '👥',
  '🗣',
  '👶',
];

const List<String> _widgetEmojisAnimals = [
  '🐶',
  '🐱',
  '🐭',
  '🐹',
  '🐰',
  '🐻',
  '🐼',
  '🐨',
  '🐯',
  '🦁',
  '🐮',
  '🐷',
  '🐽',
  '🐸',
  '🐵',
  '🙈',
  '🙉',
  '🙊',
  '🐒',
  '🦍',
  '🐔',
  '🐧',
  '🐦',
  '🐤',
  '🐣',
  '🐥',
  '🐺',
  '🦊',
  '🐗',
  '🐴',
  '🦓',
  '🦒',
  '🦌',
  '🦄',
  '🐝',
  '🐛',
  '🦋',
  '🐌',
  '🐞',
  '🐜',
  '🦗',
  '🕷',
  '🕸',
  '🦂',
  '🐢',
  '🐍',
  '🦎',
  '🦀',
  '🦑',
  '🐙',
  '🦐',
  '🐠',
  '🐟',
  '🐡',
  '🐬',
  '🦈',
  '🐳',
  '🐋',
  '🐊',
  '🐆',
  '🐅',
  '🐃',
  '🐂',
  '🐄',
  '🐪',
  '🐫',
  '🐘',
  '🦏',
  '🐐',
  '🐏',
  '🐑',
  '🐎',
  '🐖',
  '🦇',
  '🐓',
  '🦃',
  '🕊',
  '🦅',
  '🦆',
  '🦉',
  '🐕',
  '🐩',
  '🐈',
  '🐇',
  '🐀',
  '🐁',
  '🐿',
  '🦔',
  '🐾',
  '🐉',
  '🐲',
  '🦕',
  '🦖',
];

const List<String> _widgetEmojisFoods = [
  '🍏',
  '🍎',
  '🍐',
  '🍊',
  '🍋',
  '🍌',
  '🍉',
  '🍇',
  '🍓',
  '🍈',
  '🍒',
  '🍑',
  '🍍',
  '🥥',
  '🥝',
  '🍅',
  '🥑',
  '🍆',
  '🌶',
  '🥒',
  '🥦',
  '🌽',
  '🥕',
  '🥗',
  '🥔',
  '🍠',
  '🥜',
  '🍯',
  '🍞',
  '🥐',
  '🥖',
  '🥨',
  '🥞',
  '🧀',
  '🍗',
  '🍖',
  '🥩',
  '🍤',
  '🥚',
  '🍳',
  '🥓',
  '🍔',
  '🍟',
  '🌭',
  '🍕',
  '🍝',
  '🥪',
  '🥙',
  '🌮',
  '🌯',
  '🍜',
  '🥘',
  '🍲',
  '🥫',
  '🍥',
  '🍣',
  '🍱',
  '🍛',
  '🍙',
  '🍚',
  '🍘',
  '🥟',
  '🍢',
  '🍡',
  '🍧',
  '🍨',
  '🍦',
  '🍰',
  '🎂',
  '🥧',
  '🍮',
  '🍭',
  '🍬',
  '🍫',
  '🍿',
  '🍩',
  '🍪',
  '🥠',
  '☕',
  '🍵',
  '🥣',
  '🍼',
  '🥤',
  '🥛',
  '🍺',
  '🍻',
  '🍷',
  '🥂',
  '🥃',
  '🍸',
  '🍹',
  '🍾',
  '🍶',
  '🥄',
  '🍴',
  '🍽',
  '🥢',
  '🥡'
];

const List<IconData> _widgetEmojisTypesIcon = [MaterialCommunityIcons.face, MaterialCommunityIcons.hand_okay, MaterialCommunityIcons.dog, MaterialCommunityIcons.food_apple];

const List<List<String>> _widgetEmojisTypesData = [_widgetEmojisSmileys, _widgetEmojisHands, _widgetEmojisAnimals, _widgetEmojisFoods];
