import 'package:flutter/material.dart';

///
///底部弹出输入框
///
class WidgetInput<T> extends PopupRoute<T> {
  final Duration tween;
  final double iconSize;
  final String? labelText;
  final String? helperText;
  final int maxLength;
  final int maxLines;
  final String inputType;
  final String defValue;
  final EdgeInsets padding;
  String _currentValue;

  WidgetInput({
    this.tween = const Duration(milliseconds: 300),
    this.iconSize = 28,
    this.labelText,
    this.helperText,
    this.maxLength = 16,
    this.maxLines = 1,
    this.inputType = '',
    this.defValue = '',
    this.padding = const EdgeInsets.only(top: 8, left: 16, bottom: 24, right: 16),
  }) : _currentValue = defValue;

  @override
  Widget buildPage(BuildContext context, Animation<double> animation, Animation<double> secondaryAnimation) {
    final themeData = Theme.of(context);
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: Column(
        children: <Widget>[
          Expanded(
            child: GestureDetector(behavior: HitTestBehavior.opaque, child: Container(color: Colors.black45), onTap: () => Navigator.pop(context)),
          ),
          Ink(
            color: themeData.cardColor,
            padding: padding,
            child: Column(
              children: <Widget>[
                Row(
                  children: <Widget>[
                    IconButton(icon: Icon(Icons.close, size: iconSize), onPressed: () => Navigator.pop(context)),
                    const Spacer(),
                    IconButton(icon: Icon(Icons.check, size: iconSize), onPressed: () => Navigator.pop(context, _currentValue)),
                  ],
                ),
                SizedBox(height: padding.top),
                TextFormField(
                  autofocus: true,
                  initialValue: defValue,
                  keyboardType: _getInputType(),
                  textInputAction: TextInputAction.done,
                  maxLength: maxLength,
                  maxLines: maxLines,
                  decoration: InputDecoration(
                    labelText: labelText,
                    helperText: helperText,
                  ),
                  onChanged: (value) => _currentValue = value,
                  onEditingComplete: () => Navigator.pop(context, _currentValue),
                )
              ],
            ),
          )
        ],
      ),
    );
  }

  TextInputType _getInputType() {
    switch (inputType) {
      case 'number':
        return TextInputType.number;
      case 'phone':
        return TextInputType.phone;
      case 'url':
        return TextInputType.url;
      case 'emailAddress':
        return TextInputType.emailAddress;
      case 'datetime':
        return TextInputType.datetime;
      default:
        return TextInputType.text;
    }
  }

  @override
  Duration get transitionDuration => tween;

  @override
  Color get barrierColor => Colors.transparent;

  @override
  bool get barrierDismissible => true;

  @override
  String get barrierLabel => '';
}
