import 'package:flutter/material.dart';

///
///加载进度框
///
class WidgetLoading extends StatelessWidget {
  final String msg;
  final bool toast;
  final double raduis;
  final double padding;
  final double iconSize;

  const WidgetLoading({
    Key? key,
    this.msg = 'Loading',
    required this.toast,
    this.raduis = 8,
    this.padding = 8,
    this.iconSize = 50,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Material(
      type: MaterialType.transparency,
      child: Container(
        padding: toast ? EdgeInsets.all(padding) : EdgeInsets.all(padding * 2),
        margin: EdgeInsets.all(padding * 4),
        constraints: toast ? null : BoxConstraints(minWidth: iconSize * 2.5, minHeight: iconSize * 2.5),
        decoration: ShapeDecoration(
          color: themeData.brightness == Brightness.light ? (toast ? Colors.black : Colors.white) : (toast ? Colors.white : Colors.black),
          shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(toast ? raduis * 0.5 : raduis))),
        ),
        child: toast
            ? Text(
                msg,
                style: themeData.textTheme.bodyText1?.copyWith(color: themeData.brightness == Brightness.light ? Colors.white : Colors.black),
                textAlign: TextAlign.center,
              )
            : Column(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: <Widget>[
                  SizedBox(height: padding),
                  SizedBox(
                    width: iconSize,
                    height: iconSize,
                    child: CircularProgressIndicator(color: themeData.colorScheme.secondary),
                  ),
                  SizedBox(height: padding * 2.5),
                  Text(
                    msg,
                    style: themeData.textTheme.bodyText1?.copyWith(color: themeData.brightness == Brightness.light ? Colors.black : Colors.white),
                  ),
                ],
              ),
      ),
    );
  }
}
