import 'package:flutter/material.dart';
import 'package:qr_flutter/qr_flutter.dart';

///
///二维码显示
///
class WidgetQrcode extends StatelessWidget {
  final String title;
  final String data;
  final Icon icon;
  final String nick;
  final String hint;
  final Widget? logo;
  final EdgeInsets boxPadding;
  final double bixRadius;
  final double qrWidth;

  const WidgetQrcode({
    Key? key,
    required this.title,
    required this.data,
    required this.icon,
    required this.nick,
    required this.hint,
    this.logo,
    this.boxPadding = const EdgeInsets.symmetric(vertical: 10, horizontal: 20),
    this.bixRadius = 10.0,
    this.qrWidth = 260,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    final backgroundColor = themeData.brightness == Brightness.light ? Colors.grey[200] : null; //增强light状态white气泡的对比度
    return Scaffold(
      extendBodyBehindAppBar: true,
      appBar: AppBar(title: Text(title), backgroundColor: backgroundColor),
      backgroundColor: backgroundColor,
      body: Center(
        child: Container(
          padding: boxPadding,
          decoration: ShapeDecoration(
            color: Colors.white,
            shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(bixRadius))),
          ),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              SizedBox(
                width: qrWidth,
                child: Row(
                  children: [
                    icon,
                    Expanded(
                      child: Text(nick, style: themeData.textTheme.subtitle1?.copyWith(color: Colors.black), overflow: TextOverflow.ellipsis),
                    ),
                  ],
                ),
              ),
              SizedBox(height: boxPadding.bottom),
              Stack(
                alignment: Alignment.center,
                children: [
                  QrImage(
                    data: data,
                    size: qrWidth,
                    version: 4,
                    padding: EdgeInsets.zero,
                  ),
                  logo ?? const SizedBox(),
                ],
              ),
              SizedBox(height: boxPadding.top),
              Text(hint, style: themeData.textTheme.caption?.copyWith(color: Colors.black54)),
            ],
          ),
        ),
      ),
    );
  }
}
